@@ -5,8 +5,12 @@
 ## Written by Luca Scrucca                                                 ##
 #############################################################################
 
-msir <- function(x, y, nslices = msir.nslices, slice.function = msir.slices, 
-                 modelNames = NULL, G = NULL, cov = c("mle", "regularized"), 
+msir <- function(x, y, 
+                 nslices = msir.nslices, 
+                 slice.function = msir.slices, 
+                 modelNames = NULL, 
+                 G = NULL, 
+                 cov = c("mle", "regularized"), 
                  ...)
 { 
   call <- match.call()
@@ -16,14 +20,14 @@ msir <- function(x, y, nslices = msir.nslices, slice.function = msir.slices,
     stop("nslices must be an integer value or a function")
   if(!is.function(slice.function)) 
     stop("slice.function must be a function")
-  #-----------------------------------------------------------------  
+  #
   xname <- deparse(substitute(x))
   x <- as.matrix(x)
   n <- nrow(x)
   p <- ncol(x)
   if(is.null(colnames(x))) 
      colnames(x) <- paste(xname, 1:p, sep="")
-  #-----------------------------------------------------------------  
+  #
   if(length(y) != n)
     stop("Dimension of y and x does not match!")
   if(is.character(y))
@@ -37,14 +41,14 @@ msir <- function(x, y, nslices = msir.nslices, slice.function = msir.slices,
   slice.info <- slice.function(y, nslices) 
   nslices <- slice.info$nslices
   ysl <- slice.info$slice.indicator
-  #-----------------------------------------------------------------  
+  #
   # overall parameters
   tau <- slice.info$slice.sizes/n
   mu <- colMeans(x)
   # Sigma <- var(x)*(n-1)/n
   # SVD <- svd(Sigma)
   # inv.sqrt.Sigma <- crossprod(t(SVD$v)*SVD$d^(-1/4))
-  #-----------------------------------------------------------------  
+  #
   # mixture model parameters
   if(is.null(G)) 
     { # guarantees at least 10 obs per slice, 
@@ -143,9 +147,10 @@ msir.fit <- function(data, labels, G = NULL, modelNames = NULL,
 {
   mc <- match.call(expand.dots = TRUE)
   mc[[1]] <- as.name("mclustBIC")
-  mc$labels <- mc$verbose <- mc$noise <- NULL
+  mc$labels <- mc$noise <- NULL
   mc$G <- mc$modelNames <- NULL
   mc$control <- control
+  mc$verbose <- verbose
   dimData <- dim(data)
   oneD <- is.null(dimData) || length(dimData[dimData > 1]) == 1
   if(!oneD && length(dimData) != 2) 
@@ -227,10 +232,9 @@ msir.fit <- function(data, labels, G = NULL, modelNames = NULL,
 
 print.msir <- function(x, ...)
 {
-  object <- x
-  cat("Model-based SIR\n")
-  cat("\nCall:\n")
-  cat(paste(deparse(object$call), sep="\n", collapse = "\n"), "\n", sep="")
+  catwrap(paste0("\'", class(x)[1], "\' model object: "))
+  str(x, max.level = 1, give.attr = FALSE, strict.width = "wrap")
+  invisible()
 }
 
 summary.msir <- function(object, numdir = object$numdir, std = FALSE, verbose = TRUE, ...)
@@ -248,7 +252,7 @@ summary.msir <- function(object, numdir = object$numdir, std = FALSE, verbose =
      { m <- object$mixmod[[i]]
        sizes[[i]] <- as.vector(table(m$classification)) }
   tab <- rbind(tab, sapply(sizes, function(s) paste(s, collapse="|")))
-  rownames(tab) <- c("Mixt.Mod.", "N.comp.", "N.obs.")
+  rownames(tab) <- c("GMM", "Num.comp.", "Num.obs.")
   
   basis <- object$basis[,seq(numdir),drop=FALSE]
   std.basis <- object$std.basis[,seq(numdir),drop=FALSE]
@@ -257,21 +261,6 @@ summary.msir <- function(object, numdir = object$numdir, std = FALSE, verbose =
   evalues <- rbind("Eigenvalues" = evalues, 
                    "Cum. %" = cumsum(evalues/sum(object$evalues))*100)
   colnames(evalues) <- colnames(basis)
-  #  { se <- msir.dirse(object)
-  #    out <- matrix(as.double(NA), nrow(basis), numdir*3)
-  #    for(j in 1:numdir)
-  #       { out[,((j-1)*3+1):((j-1)*3+3)] <- 
-  #                        cbind(basis[,j], se[,j], basis[,j]/se[,j])
-  #       }
-  #    rownames(out) <- rownames(basis)
-  #    cnames <- rep(NA, numdir*3)
-  #    cnames[(0:(ncol(basis)-1))*3+1] <- colnames(basis)
-  #    cnames[(0:(ncol(basis)-1))*3+2] <- "SE"
-  #    cnames[(0:(ncol(basis)-1))*3+3] <- "Est/SE"
-  #    colnames(out) <- cnames
-  #    cat("\nEigenvectors and approximate standard errors:\n")
-  #    print(out, digits = digits)
-  #  }
 
   StructDimTab <- NULL
   if(!is.null(object$bic))
@@ -298,11 +287,12 @@ summary.msir <- function(object, numdir = object$numdir, std = FALSE, verbose =
 
 print.summary.msir <- function(x, digits = max(5, getOption("digits") - 3), ...)
 {
-  cat("\nCall:\n")
-  cat(paste(deparse(x$call), sep="\n", collapse = "\n"), "\n\n", sep="")
-  cat("Model-based SIR\n\n")
-  #
-  cat("Slices:\n")
+  txt <- paste(rep("-", min(50, getOption("width"))), collapse = "")
+  catwrap(txt)
+  catwrap("Model-based SIR")
+  catwrap(txt)
+  
+  cat("\nSlices:\n")
   print(x$tab, na.print="", quote=FALSE)
 
   if(x$verbose)
@@ -325,272 +315,26 @@ print.summary.msir <- function(x, digits = max(5, getOption("digits") - 3), ...)
   invisible()
 }
 
-plot.msir <- function(x, which, type = c("pairs", "2Dplot", "spinplot", "evalues", "coefficients"), span = NULL, std = TRUE, ylab, xlab, restore.par = TRUE, ...)
-{
-  object <- x  # Argh.  Really want to use 'object' anyway
-  y <- object$y
-  numdir <- object$numdir
-  if(missing(which)) which <- 1:numdir
-  dir <- object$dir[,which,drop=FALSE]
-  type <- match.arg(type)
-  if(type != "spinplot" & isTRUE(restore.par))
-    { oldpar <- par(no.readonly = TRUE)
-      on.exit(par(oldpar)) }
-  switch(type,
-   "pairs" = { if(is.numeric(span))
-                 { pairs(data.frame(y, dir), gap = 0,
-                         upper.panel = 
-                           function(...)
-                            panel.loess(col.smooth = "green3", span = span, ...),
-                         ...) }
-              else  pairs(data.frame(y, dir), gap = 0, ...) },
-   "2Dplot" = { plot(dir[,1], y, 
-                     xlab = ifelse(missing(xlab), paste("MSIR Dir", which[1], sep=""), xlab), 
-                     ylab = ifelse(missing(ylab), "Y", ylab), ...) 
-                if(is.numeric(span))
-                  { lines(l <- loess.sd(dir[,1], y, k = 1, span = span))
-                    lines(l$x, l$upper, lty=2)
-                    lines(l$x, l$lower, lty=2) } },
-   "spinplot" = { if(is.element("rgl", installed.packages()[,1]))
-                    { data <- data.frame(dir[,1], y, dir[,2])
-                      colnames(data) <- c(colnames(dir)[1], "y", colnames(dir)[2])
-                      if(!missing(ylab)) colnames(data)[2] <- ylab
-                      if(!missing(xlab)) colnames(data)[c(1,3)] <- xlab
-                      spinplot(data, fit.smooth = is.numeric(span), span = span, ...)
-                    }
-                },
-   "evalues" = { bic <- msir.bic(object, plot = FALSE)
-                 oldpar <- par(mfrow=c(1,2), mar = c(5,4,2,0.5))
-                 on.exit(par(oldpar))
-                 plot(1:length(object$evalues), object$evalues, 
-                      type = "b", xaxt = "n", panel.first = grid(),
-                      xlab = "Dimension", ylab = "Eigenvalues")
-                 axis(1, at = 1:length(object$evalues))
-                 if(!is.null(object$permtest))
-                   { pval <- object$permtest$summary[,2]
-                     pvalcol <- cut(pval, c(0,0.05,0.10,1), 
-                                    include.lowest = TRUE,
-                                    labels = c("black", "grey70", "white"))
-                     points(1:length(object$evalues), object$evalues,
-                            pch = 21, bg = paste(pvalcol))
-                     legend("topright", inset = 0.01, 
-                            title = "p-value",
-                            pch = 21, pt.bg = paste(levels(pvalcol)),
-                            legend = c("[0-0.05)", "[0.05-0.10)", "[0.10-1]"))
-                     }
-                 par(mar = c(5,0.5,2,4))
-                 plot(0:(length(bic$crit)-1), bic$crit-max(bic$crit), 
-                         type = "b", xlab = "Dimension", 
-                         xaxt = "n", yaxt = "n",
-                         panel.first = abline(h = -c(0,2,6,10), 
-                                              v = 0:(length(bic$crit)-1),
-                                              lty = 3, col = "grey"))
-                 axis(1, at = 0:(length(bic$crit)-1))       
-                 axis(4)
-                 mtext(expression(paste(Delta, "BIC-type criterion", sep="")),
-                       side = 4, line = 2.5)
-                 points(bic$d, 0, pch = 20)
-               },
-    "coefficients" = { if(missing(which)) which <- 1:object$numdir
-                       if(std) B <- object$std.basis[,which,drop=FALSE]
-                       else    B <- object$basis[,which,drop=FALSE]
-                       evalues <- object$evalues[which]
-      ev  <- signif(evalues, 4)
-      evp <- round(evalues/sum(object$evalues[1:numdir])*100,2)
-      if(length(evalues) != ncol(B))
-        stop("number of columns of coef must be equal to num. of eigenvalues!")
-      w = max(strwidth(rownames(B), units="inches"))*1.1
-      ww = c(w,rep((par("fin")[1]-w)/ncol(B),ncol(B)))
-      ww = pmax(0.1, ww/sum(ww))
-      layout(matrix(1:(ncol(B)+1), 1, ncol(B)+1), 
-             widths = ww, heights = rep(1,ncol(B)+1))
-      mar <- par()$mar
-      par(mar = c(mar[1], 0, mar[3], 1))
-      plot(0, 0, type="n", ylim=c(1,nrow(B)), 
-           xlab="", ylab="", axes=FALSE)
-      mtext(rownames(B), side=4, at=nrow(B):1, las=1, adj=1, cex=0.8)
-      par(mar = c(mar[1], 1, mar[3], 1))
-      for(j in 1:ncol(B))
-         { plot(0, 0, type="n", 
-                xlim = range(0,B[,j])*1.1,
-                ylim = c(1,nrow(B)),
-                xlab ="", ylab="", yaxt="n")
-            mtext(colnames(B)[j], side=3, line=1)
-            mtext(paste(ev[j], " (", evp[j], "%)", sep=""), 
-                  side=1, line=3, cex = 0.8)
-            for(i in 1:nrow(B))
-               lines(c(0,B[i,j]), rep(nrow(B)+1-i,2), lwd=2)
-            abline(v=0) 
-         }
-     }
-  )
-  invisible()
-}
+# msir.dir <- function(object, numdir = object$numdir)
+# { 
+#   if(class(object) != "msir")
+#     stop("object is not of class 'msir'")
+#   object$dir[,1:numdir]  
+# }
 
-# temporary patch waiting spinplot to be made available on CRAN
-spinplot <- function(data, ...) plot3d(data[,c(1,3,2)], type = "p", ...)
-
-
-msir.dir <- function(object, numdir = object$numdir)
-{ 
-  if(class(object) != "msir")
-    stop("object is not of class 'msir'")
-  object$dir[,1:numdir]  
-}
-
-## Assessing dimension
-
-msir.permutation.test <- function(object, npermute = 99, numdir = object$numdir, verbose = TRUE)
-{
-  test.statistic <- function(object, nd)
-    { length(object$y) * rev(cumsum(rev(object$evalues)))[1:nd] }
-  call <- object$call
-  n <- length(object$y)
-  z <- scale(object$x, center = TRUE, scale = FALSE) %*% object$evectors
-  nd <- min(numdir,length(which(abs(object$evalues)>1.e-8))-1)
-  nt   <- nd + 1
-  obstest <- test.statistic(object, nt)
-  count <- val <- rep(0, nt)
-  ysl <- object$slice.info$slice.indicator
-  nslices <- object$slice.info$nslices
-  G <- lapply(object$mixmod, function(m) m$G)
-  modelNames <- lapply(object$mixmod, function(m) m$modelName)
-  call$G <- G
-  call$modelNames <- modelNames
-  #
-  if(verbose)
-    { cat("Calculating, please be patient...\n")
-      flush.console()
-      pbar <- txtProgressBar(min = 0, max = npermute, style = 3) }
-  #
-  for(j in 1:npermute) 
-     { perm <- sample.int(n)
-       for(col in 0:nd)
-          { xx <- if(col == 0) 
-                    z[perm,] else cbind(z[,(1:col)], z[perm,-(1:col)])
-            pmod <- msir(xx, ysl, nslices = nslices, 
-                         G = G, modelNames = modelNames)
-            val[col + 1] <- test.statistic(pmod, col + 1)[col + 1]
-          }
-       val[is.na(val)] <- Inf
-       count[val > obstest] <- count[val > obstest] + 1
-       if(verbose) setTxtProgressBar(pbar, j)
-     }
-  if(verbose) { close(pbar)
-                cat("\n") }
-  pval <- (count)/(npermute + 1)
-  ans <- as.data.frame(cbind(obstest, pval))
-  rownames(ans) <- paste(0:(nt - 1), "D vs >= ", 1:nt, "D", sep = "")
-  colnames(ans) <- c("Stat", "p-value")
-  ans <- list(summary = ans, npermute = npermute)
-  # assign ans to object in parent environment
-  objname <- deparse(substitute(object))
-  object$permtest <- ans
-  assign(objname, value = object, 
-         envir = sys.frame(sys.parent()))
-  #
-  return(ans)
-}
-
-##
-## BIC-type criterion
-##
-
-msir.bic <- function(object, type = 1, plot = FALSE)
-{
-  svd <- svd(object$sigma, nu=0)
-  inv.sqrt.sigma <- svd$v %*% diag(1/sqrt(svd$d)) %*% t(svd$v)
-  M <- t(inv.sqrt.sigma) %*% object$M %*% inv.sqrt.sigma
-  out <- bicDimRed(M, x = object$x, nslices = length(object$f), type = type)
-  if(plot)
-    { oldpar <- par(mfrow=c(1,2), mar = c(5,4,2,0.5))
-      on.exit(par(oldpar))
-      plot(1:length(object$evalues), object$evalues, 
-           type = "b", xaxt = "n", panel.first = grid(),
-           xlab = "Dimension", ylab = "Eigenvalues")
-      axis(1, at = 1:length(object$evalues))       
-      par(mar = c(5,0.5,2,4))
-      plot(0:(length(out$crit)-1), out$crit, 
-           type = "b", panel.first = grid(),
-           xlab = "Dimension", xaxt = "n", yaxt = "n")
-      axis(1, at = 0:(length(out$crit)-1))       
-      axis(4)
-      mtext("BIC-type criterion", side = 4, line = 2.5)
-      points(out$d, max(out$crit), pch = 20)
-    }
-  # assign ans to object in parent environment
-  objname <- deparse(substitute(object))
-  object$bic <- out
-  assign(objname, value = object, 
-         envir = sys.frame(sys.parent()))
-  #
-  return(out)  
-}
-
-bicDimRed <-function(M, x, nslices, type = 1, tol = sqrt(.Machine$double.eps)) 
-{
-# BIC-type criterion for the determination of the structural dimension of
-# a dimension reduction method.
-# The method selects d as the maximizer of
-#
-#   G(d) = logL(d) - Penalty(p,d,n)
-#
-# Arguments:
-# M = the kernel matrix
-# x = the matrix of data
-# nslices = number of slices used for slicing y
-# type = penalty term used:
-#        1)    -(p-d)*log(n)
-#        2-3)  0.5*C*d*(2*p-d+1))
-#            where C is defined as
-#            2) (0.5*log(n) + 0.1*n^(1/3))/2 * nslices/n
-#            3) log(n) * nslices/n 
-#        4)    1/2*d*log(n)
-#
-# Returns a list with elements:
-# d = the selected dimension 
-# crit = the values of the criterion
-# evalues = eigenvalues of the kernel matrix M
-#
-# Reference (penalty 2-3):
-# Zhu, Miao and Peng (2006) "Sliced Inverse Regression for CDR Space
-#   Estimation", JASA.
-# Reference (penalty 1):
-# Zhu, Zhu (2007) "On kernel method for SAVE", Journal of Multivariate Analysis
-#
-   M <- as.matrix(M)
-   x <- as.matrix(x)
-   n <- nrow(x)
-   p <- ncol(x)
-   h <- nslices
-   # eigen decomposition
-   ev <- svd(M)$d
-   ev <- (ev+abs(ev))/2
-   # BIC-type criterion
-   ev2 <- svd(M + diag(p))$d
-   ev2 <- (ev2+abs(ev2))/2
-   # penalty term
-   penalty <- function(d, type = 1)
-   { 
-     pen <- 0
-     if(type == 4) pen <- 1/2*d*log(n)
-     else if(type == 1) pen <- -(p-d)*log(n)
-     else { if(type == 2) C <- (0.5*log(n) + 0.1*n^(1/3))/2 * h/n
-            if(type == 3) C <- log(n) * h/n 
-            pen <- 0.5*C*d*(2*p-d+1)
-          }
-     return(pen)      
-   }
-   k <- sum(ev > tol)
-   crit <- l <- rep(0, k)
-   for(d in 0:(k-1)) 
-      { l[d+1] <- 0.5*n*sum((log(ev2)+1-ev2)[(1+min(k,d)):p])
-        crit[d+1] <- l[d+1] - penalty(d, type) }
-   dmax <- which.max(crit) - 1
-   names(crit) <- paste("d=", 0:(k-1), sep="")
-   # return
-   ans <-list(evalues = ev[1:k], l = l[1:k], crit = crit, d = dmax)
-   return(ans)
+predict.msir <- function(object, dim = 1:object$numdir, newdata, ...)
+{  
+  dim <- dim[dim <= object$numdir]
+  if(missing(newdata))
+  { 
+    dir <- object$dir[,dim,drop=FALSE] 
+  } else 
+  { 
+    newdata <- as.matrix(newdata) 
+    newdata <- scale(newdata, center = colMeans(object$x), scale = FALSE)
+    dir <- newdata %*% object$basis[,dim,drop=FALSE]
+  } 
+  return(dir)
 }
 
 eigen.decomp <- function(X1, X2, inv = FALSE, tol = sqrt(.Machine$double.eps))
@@ -724,7 +468,7 @@ msir.nslices <- function(n, p)
   pmax(3, floor(log2(n/sqrt(p))))
 }
 
-## slicing functions in package 'dr' version. 3.x.x by Sanford Weisberg
+## slicing functions in package 'dr' version. 3.0.x by Sanford Weisberg
 msir.slices <- function(y, nslices)
 {
     dr.slice.1d <- function(y, h) {
@@ -742,32 +486,66 @@ msir.slices <- function(y, nslices)
         }
         list(slice.indicator = z, nslices = length(u), slice.sizes = sizes)
     }
-    dr.slice2<-function(y,h)
-		{
-		  or <- order(y)
-		  n <- length(y)
-		  m<-floor(n/h)
-		  r<-n-m*h
-		  start<-sp<-ans<-0
-		  j<-1
-		  while((start+m)<n) {
-		      if (r==0)
-		        start<-start
-		      else
-		        {start<-start+1
-		         r<-r-1
-		        }
-		       while (y[or][start+m]==y[or][start+m+1])
-		          start<-start+1
-		       sp[j]<-start+m
-		       start<-sp[j]
-		       j<-j+1
-		  }
-		  sp[j]<-n
-		  ans[or[1:sp[1]]] <- 1
-		  for (k in 2:j){ans[ or[(sp[k-1]+1):sp[k] ] ] <- k}
-		  list(slice.indicator=ans, nslices=j, slice.sizes=c(sp[1],diff(sp)))
-		}
+#     old version
+#     dr.slice2<-function(y,h)
+# 		{
+# 		  or <- order(y)
+# 		  n <- length(y)
+# 		  m <- floor(n/h)
+# 		  r <- n-m*h
+# 		  start <- sp <- ans <-0
+# 		  j <- 1
+# 		  while((start+m)<n) {
+# 		      if (r==0)
+# 		        start<-start
+# 		      else
+# 		        {start<-start+1
+# 		         r<-r-1
+# 		        }
+# 		       while (y[or][start+m]==y[or][start+m+1])
+# 		          start<-start+1
+# 		       sp[j]<-start+m
+# 		       start<-sp[j]
+# 		       j<-j+1
+# 		  }
+# 		  sp[j]<-n
+# 		  ans[or[1:sp[1]]] <- 1
+# 		  for (k in 2:j){ans[ or[(sp[k-1]+1):sp[k] ] ] <- k}
+# 		  list(slice.indicator=ans, nslices=j, slice.sizes=c(sp[1],diff(sp)))
+# 		}
+    dr.slice2 <- function(y,h)
+    {
+       myfind <- function(x,cty) 
+       {
+          ans<-which(x <= cty)
+          if (length(ans)==0) length(cty) else ans[1]
+       } 
+       or <- order(y)     # y[or] would return ordered y
+       cty <- cumsum(table(y))  # cumulative sums of counts of y
+       names(cty) <- NULL # drop class names
+       n <- length(y)     # length of y
+       m<-floor(n/h)      # nominal number of obs per slice
+       sp <- end <- 0     # initialize
+       j <- 0             # slice counter will end up <= h
+       ans <- rep(1,n)    # initialize slice indicator to all slice 1
+       while(end < n-2)   # find slice boundaries: all slices have at least 2 obs
+       { 
+          end <- end+m
+          j <- j+1       
+          sp[j] <- myfind(end,cty) 
+          end <- cty[sp[j]]
+       }
+       sp[j] <- length(cty)
+       for (j in 2:length(sp)) # build slice indicator
+       { 
+         firstobs <- cty[sp[j-1]]+1
+         lastobs <- cty[sp[j]]
+         ans[or[firstobs:lastobs]] <- j
+       }
+       list(slice.indicator = ans, nslices = length(sp),
+            slice.sizes = c(cty[sp[1]],diff(cty[sp])))
+    }
+    
     p <- if (is.matrix(y)) dim(y)[2] else 1
 	  h <- if (length(nslices) == p) nslices else rep(ceiling(nslices^(1/p)),p)
 	  a <- dr.slice.1d( if(is.matrix(y)) y[,1] else y, h[1])
@@ -825,37 +603,3 @@ msir.componentsSlice <- function(object)
 }
 
 
-##############################################################################
-## Auxiliary functions
-
-mvdnorm <- function(x, mean, sigma, log = FALSE, tol = sqrt(.Machine$double.eps))
-{
-# Multivariate normal probability density function (pdf)
-  if(is.vector(x)) 
-    { x <- matrix(x, ncol = length(x)) }
-  else
-    { x <- as.matrix(x) }
-  if(missing(mean)) 
-    { mean <- rep(0, length = ncol(x)) }
-  if(missing(sigma)) 
-    { sigma <- diag(ncol(x)) }
-  #    
-  SVD <- svd(sigma)
-  pos <- (SVD$d > max(tol*SVD$d[1], 0)) # in case of not full rank covar matrix
-  inv.sigma <- SVD$v[,pos,drop=FALSE] %*% (1/SVD$d[pos] *
-                                          t(SVD$u[,pos,drop=FALSE]))
-  md <- mahalanobis(x, center = mean, cov = inv.sigma, inverted = TRUE)
-  logdet <- sum(log(SVD$d[pos]))
-  logdens <- -0.5*(ncol(x) * log(2 * pi) + logdet + md)
-  if(log) return(logdens)
-  else    return(exp(logdens))
-}
-
-normalize <- function(x)
-{
-# Normalize the vector x to have unit length
-  x <- as.vector(x)
-  x <- x/sqrt(crossprod(x))
-  return(x)
-}
-
